public class Board
//////////////////
{
  // -----------------------------------------------------------------

  /**
    Board side size
  */
  private int sideSize;

  /**
    Holds the row values where are allocated the queens in
    column [0..length-1].
    -1 if there is no allocation for that column.
    col, row are in [0..length-1]

    If there is allocation for column c, it means that the column c is
    attacked.
  */
  private byte[] rows;

  /**
    Stores which rows are currently under attack
  */
  private boolean[] rowsAttacked;

  /**
    Stores which diagonals are currently under attack.
    The index in the array of the diagonal passing through the
    cell (row,col) is: (col + row)
  */
  private boolean[] diagsAttacked;

  /**
    Stores which antidiagonals are currently under attack
    The index in the array of the antidiagonal passing through the
    cell (row,col) is: ( col - row + sideSize - 1 )
  */
  private boolean[] antiDiagsAttacked;
  // -----------------------------------------------------------------


  public Board( int sideSize )
  ////////////////////////////
  {
    this.sideSize = sideSize;
    rows = new byte[ sideSize ];
    rowsAttacked = new boolean[ sideSize ];
    diagsAttacked = new boolean[ 2 * sideSize - 1 ];
    antiDiagsAttacked = new boolean[ 2 * sideSize - 1 ];

    // Initialize.
    for ( int i = 0; i < sideSize; i++ )
    {
      rows[ i ] = -1;             // no allocations
      rowsAttacked[ i ] = false;  // no row attacked
    }

    for ( int i = 0; i < diagsAttacked.length; i++ )
    {
      diagsAttacked[ i ] = false;  // no diag attacked
      antiDiagsAttacked[ i ] = false;  // no antiDiag attacked
    }
  }


  /**
    Copy constructor
  */
  public Board( Board source )
  ////////////////////////////
  {
    sideSize = source.sideSize;
    rows = new byte[ sideSize ];
    rowsAttacked = new boolean[ sideSize ];
    diagsAttacked = new boolean[ 2 * sideSize - 1 ];
    antiDiagsAttacked = new boolean[ 2 * sideSize - 1 ];

    System.arraycopy( source.rows, 0, rows, 0, rows.length );
    System.arraycopy( source.rowsAttacked, 0, rowsAttacked, 0, rowsAttacked.length );
    System.arraycopy( source.diagsAttacked, 0, diagsAttacked, 0, diagsAttacked.length );
    System.arraycopy( source.antiDiagsAttacked, 0, antiDiagsAttacked, 0, antiDiagsAttacked.length );
  }


  /**
    Informs if the cell at (row, col) is attacked by any queen, i.e:
      - same row than an existing allocation
      - same col than an existing allocation
      - same diagonal or antidiagonal than an existing allocation:
        - col + row == c + rows[ c ]
        - col - row == c - rows[ c ]
  */
  final boolean isAttacked( int row, int col )
  ////////////////////////////////////////////
  {
    return  rowsAttacked[ row ] ||
            diagsAttacked[ col + row ] ||
            antiDiagsAttacked[ col - row + sideSize - 1 ];
  }



  /**
    Same as isAttacked(), but slower.
    Doesn't use the structures
      rowsAttacked[]
      diagsAttacked[]
      antiDiagsAttacked[]
  */
  final boolean isAttacked_2( int row, int col )
  ////////////////////////////////////////////
  {
    boolean attacked = false;
    for ( int c = 0; c < rows.length && !attacked; c++ )
    {
      if ( rows[ c ] != -1 )
      {
        if ( row == rows[ c ] )
          attacked = true;

        else if ( col + row == c + rows[ c ] )
          attacked = true;

        else if ( col - row == c - rows[ c ] )
          attacked = true;
      }
    }
    return attacked;
  }


  /**
    Returns the row where is allocated the queen in column c.
    -1 if there is no allocation.
    col is in [0..sideSize-1]
    return value is in [0..sideSize-1]
  */
  final int getQueenRow( int col )
  ////////////////////////////////
  {
    return rows[ col ];
  }


  final void setQueenRow( int col, int row )
  //////////////////////////////////////////
  {
    // Restore values
    if ( rows[ col ] != -1 )
    {
      rowsAttacked[ rows[ col ] ] = false;
      diagsAttacked[ col + rows[ col ] ] = false;
      antiDiagsAttacked[ col - rows[ col ] + sideSize - 1 ] = false;
    }

    rows[ col ] = (byte)row;
    rowsAttacked[ row ] = true;
    diagsAttacked[ col + row ] = true;
    antiDiagsAttacked[ col - row + sideSize - 1 ] = true;
  }


  final void cleanQueenCol( int col )
  ///////////////////////////////////
  {
    if ( rows[ col ] != -1 )
    {
      rowsAttacked[ rows[ col ] ] = false;
      diagsAttacked[ col + rows[ col ] ] = false;
      antiDiagsAttacked[ col - rows[ col ] + sideSize - 1 ] = false;
      rows[ col ] = -1;
    }
  }


  final int getSideSize()
  ///////////////////////
  {
    return sideSize;
  }


  public String toString()
  ////////////////////////
  {
    StringBuffer ret = new StringBuffer( "[ " );
    for ( int i = 0; i < rows.length; i++ )
    {
      ret.append( rows[i] ).append( ' ' );
    }
    ret.append( ']' );
    return ret.toString();
  }

} // class Board

