//
// File:    QueensTest.java
// Author:  Jorge F. Del Teglia
// Date:    20000224qd
//


import java.io.PrintWriter;
import java.util.Date;
import java.util.Calendar;
import java.text.SimpleDateFormat;


/**
  Provides a simple solution to the problem of the queens:
    given a chess board of side size n, find all possible ubications
    for n queens in such a way that no one attacks other.

  Running the command line program without parameters, will try to
  solve for several side sizes.
  Callin g it with a single integer parameter will try to solve that
  case.
  In any case, will measure the time took to solve each problem.

  Comment: the initial implementation was more "functional" oriented.
  But the performance was very bad. That version used the classes

    BoardCollection

  and replaced the method doQueensAllocationFrom(.) by
  getQueensAllocationsFrom. The code was as following:

    static BoardCollection getQueenAllocationsFrom( int queenCol, Board b )
    {
      BoardCollection results = new BoardCollection(); // Results holder
      Board bCopy = new Board( b );  // Working copy
      final int N = b.getSideSize();

      for ( int row = 0; row < N; row++ )
      {
        if ( !bCopy.isAttacked( row, queenCol ) )
        {
          bCopy.setQueenRow( queenCol, row );
          if ( queenCol == N - 1 ) // recursion base
            results.add( bCopy );
          else // recursion
            results.addAll( getQueenAllocationsFrom( queenCol + 1, bCopy ) );
        } // if
      } // for row
      return results;
    }
*/
public class QueensTest
///////////////////////
{
  private static final boolean  OUTPUT_SOLUTIONS = false;
  private static PrintWriter    outputStream = null;

  /**
    Finds all Boards that are solution, based in Board b
    (already allocated to queen in column queenCol-1).
    It is, tries all valid rows for queen queenCol and then
    calls itself recursively to allocate the other columns.

    If the constant OUTPUT_SOLUTIONS is defined as true, will output
    each found solution to the stream referenced by outputStream

    Returns the number of solutions found. 0 if no one was found.
  */
  static long doQueenAllocationFrom( int queenCol, Board board )
  //////////////////////////////////////////////////////////////
  {
    final int N = board.getSideSize();

    long nSolutions = 0;
    for ( int row = 0; row < N; row++ )
    {
      if ( !board.isAttacked( row, queenCol ) )
      {
        board.setQueenRow( queenCol, row );
        if ( queenCol == N - 1 ) // recursion base
        {
          if ( OUTPUT_SOLUTIONS )
            outputStream.println( board );
          nSolutions++;
        }
        else // recursion
          nSolutions += doQueenAllocationFrom( queenCol + 1, board );
      } // if
    } // for row

    // Clean value setted in setQueenRow, if necessary.
    board.cleanQueenCol( queenCol );

    return nSolutions;
  }


  static void doIndividualTest( int sideSize )
  ////////////////////////////////////////////
  {
    long initialTime  = new java.util.Date().getTime();
    long nSolutions   = doQueenAllocationFrom( 0, new Board( sideSize ) );
    long duration     = new java.util.Date().getTime() - initialTime;

    String msg = "Side=" + sideSize + ", [" + duration + " ms]: " +
                 nSolutions + " solutions";
    if ( nSolutions > 0 )
      outputStream.println( msg );
    else
      outputStream.println( msg + ": couldn't be solved!" );
    if ( OUTPUT_SOLUTIONS )
      outputStream.println( "--------------------------------------------------" );
    outputStream.flush();
  }


  static void doIncrementalTest()
  ///////////////////////////////
  {
    for ( int sideSize = 1; true; sideSize++ )
      doIndividualTest( sideSize );
  }


  public static void main( String[] args )
  ////////////////////////////////////////
  {
    // Prepare parameters
    int sideSize = 4; // default
    if ( args.length > 0 )
    {
      try
      {
        sideSize = Integer.parseInt( args[ 0 ] );
      }
      catch ( NumberFormatException e )
      {
        System.out.println( "Error in the format of arguments. Should be integer!" );
      }
    }

    // Specify and prepare output stream
    outputStream = new PrintWriter( System.out );
    Date now = Calendar.getInstance().getTime();
    String nowQT = new SimpleDateFormat( "yyyyMMddHHmm" ).format( now );
    outputStream.println( ">>>" );
    outputStream.println( ">>> Queens problem solution at " + nowQT + "qm" );
    outputStream.println( ">>>\n\n" );

    // Run test
    try
    {
      if ( args.length > 0 )
        doIndividualTest( sideSize );
      else
        doIncrementalTest();
    }
    catch ( OutOfMemoryError e )
    {
      outputStream.println( "I'm very sorry, but the system is out of memory!" );
      System.exit( 1 );
    }
  }

} // class QueensTest
